(function () {
  if(!String.prototype.trim) {
    String.prototype.trim = function () {
      return this.replace(/^\s+|\s+$/g,'');
    };
  }
  
  var LS = window["LS"] || {};

  var openModal = function (title, body, buttonTitle, buttonAction, options) {
    var modal, modalBody;
    $("#commonModalLabel").html(title);
    modal = $("#commonModal");
    modalBody = modal.find('.modal-body');

    if (body) {
      modalBody.html(body);
    }
    else {
      modalBody.remove();
    }
    modal.find('.btn-submit').click(buttonAction).addClass(options.buttonClass || 'btn-primary').html(buttonTitle);
    modal.modal('show');
  };

  this.removeConfirm = function (body, buttonAction) {
    openModal("Confirm", body, "Confirm", buttonAction, {
      buttonClass: 'btn-danger'
    });
  };

  $(function () {
    $(".toggle-scope").each(function () {
      var buttons = $(this).find(".opener, .closer");
      var toggleable = $(this).find(".toggleable, .anti-toggleable");
      buttons.click(function () {
        buttons.toggle();
        toggleable.toggle();
        toggleable.filter(":visible").find("textarea, input").eq(0).focus();
      });
    });

    var editableElements = "input:enabled, textarea:enabled, select:enabled";
    $(".inline-editable").each(function () {
      var block = $(this);
      var form = block.find("form");

      block.find(".edit").click(function () {
        block.addClass("edit-mode").find("input").focus();
      });

      block.find(".cancel").click(function () {
        form.get(0).reset();
        block.removeClass("edit-mode");
      });

      LS.common.ajaxFormSubmit(form, function (data) {
        $.each(data, function (name, value) {
          block.find("[data-role=" + name + "]").html(value).val(value);
        });
        block.removeClass("edit-mode").find(editableElements).removeAttr("disabled");
      }, function (resp) {
        var errors = _.pairs($.parseJSON(resp.responseText));
        form.find(".alert-error").removeClass("hide").html(errors[0][1]);
      })
    });

    setTimeout(function () {
      $("#message-line .alert-success").fadeOut(1000);
    }, 5000);
    
    $('[rel=tooltip]').parent().tooltip({
      selector: "[rel=tooltip]"
    });

    $("#edit-password").click(function () {
      if ($(this).is(":checked")) {
        $("#password").removeAttr("disabled");
      }
      else {
        $("#password").attr("disabled", "disabled");
      }
    });
    $("input[placeholder],textarea[placeholder]").each(function(){
      $(this).watermark($(this).attr("placeholder"));
    });
  });

  LS.common = {
    preloaderImage: function(){
      return $("<img>").attr("src", "/assets/images/preloader-small.gif").css("margin-left", "5px");
    },
    
    // Action performed when user clicks on the "Test connection" button. Calls the server.
    testConnection: function (button) {
      var form = button.parents("form:eq(0)");
      var formData = form.serialize();
      var clearErrors = function () {
        form.find(".control-group").removeClass("error");
        form.find(".help-inline").remove();
        button.next("img").remove(); //remove Preloader
      };
      var showMessage = function (data) {
        button.popover({
          html: true,
          trigger: 'manual',
          placement: 'bottom',
          title: "Connection Result",
          content: data
        });
        button.popover('show');
      };
      
      button.after(LS.common.preloaderImage());
      
      $.post(button.data("url"), formData, function (data) {
        clearErrors();
        showMessage(data);
      })
      .error(function (data) {
        var errors = $.parseJSON(data.responseText);
        clearErrors();
        $.each(errors, function (fieldName, error) {
          var field = form.find("[name='" + fieldName + "']");
          field.after('<span class="help-inline">' + error.join(", ") + '</span>');
          field.parents(".control-group").addClass("error");
        });
        showMessage("<span class='red'>Fields not properly filled</span>");
      });
    },

    ajaxFormSubmit: function (form, successCallback, errorCallback, onSubmit) {
      return form.unbind('submit').submit(function () {
        form.find("button").attr("disabled", "disabled");
        if (onSubmit) onSubmit();
        $.ajax({
          type: 'POST',
          url: form.attr("action"),
          data: form.serializeArray(),
          error: errorCallback,
          success: successCallback,
          complete: function () {
            form.find("button").removeAttr("disabled");
          }
        });
        return false;
      });
    }
  };
})();